#ifndef MAPA_H
#define MAPA_H

#include <stdio.h> //per printf
#include <stdlib.h> //per atoi

//----------------------------------------------------------------
// Coordenades
//----------------------------------------------------------------
struct Coordenades
{
    int i;
    int j;
};

/* Compara c1 i c2, retorna true si són iguals, false en cas contrari
 */
bool Coordenades_Igual( const Coordenades& c1, const Coordenades& c2 )
{
    return c1.i == c2.i && c1.j == c2.j;
}

/* Retorna coordenades noves resultants d'avançar un pas des de c1 en direcció a
   c2, sense permetre moviment diagonal i prioritzant moviment
   horitzontal (en j)
*/
Coordenades Coordenades_FerUnPas( const Coordenades& c1, const Coordenades& c2 )
{
    int diff_i = c2.i - c1.i;
    int diff_j = c2.j - c1.j;
    int abs_diff_i = ( diff_i < 0 ) ? -diff_i : diff_i;
    int abs_diff_j = ( diff_j < 0 ) ? -diff_j : diff_j;
    Coordenades coords;
    if( abs_diff_i == 0 && abs_diff_j == 0 )
        coords = c1;
    else if( abs_diff_i <= abs_diff_j )
    {
        coords.i = c1.i;
        coords.j = (diff_j < 0) ? c1.j-1 : c1.j+1;
    }
    else if( abs_diff_i > abs_diff_j )
    {
        coords.i = (diff_i < 0) ? c1.i-1 : c1.i+1;
        coords.j = c1.j;
    }
    return coords;
}

//----------------------------------------------------------------
// Mapa
//----------------------------------------------------------------
struct Mapa
{
    int F;
    int C;
    char* dades;
    Coordenades coords_sortida;
    Coordenades coords_joia;
};

void InicialitzarMapa( Mapa& m, int F, int C )
{
    m.F = F;
    m.C = C;
    m.dades = new char[F*C];
}

void DestruirMapa( Mapa& m )
{
    m.F = 0;
    m.C = 0;
    delete m.dades;
    m.dades = 0;
}

// Modifica els continguts de la casella (fila,columna)
void ModificarMapa( Mapa& m, int fila, int columna, char contingut )
{
    m.dades[fila*m.C + columna] = contingut;
}

// Retorna els continguts de la casella (fila,columna)
char ConsultarMapa( const Mapa& m, int fila, int columna )
{
    return m.dades[fila*m.C + columna];
}

// Retorna les coordenades de la casella que té el contingut indicat
Coordenades BuscarMapa( const Mapa& m, char contingut )
{
    Coordenades coords;
    coords.i = -1;
    coords.j = -1;
    for( int i = 0; i < m.F; i++ )
    {
        for( int j = 0; j < m.C; j++ )
        {
            if( ConsultarMapa(m,i,j) == contingut )
            {
                coords.i = i;
                coords.j = j;
                return coords;
            }
        }
    }
    return coords;
}

/* Llegeix mapa continguts del mapa de l'entrada estandar */
void LlegirMapa( Mapa& m )
{
    char c;
    // llegir files
    for( int i = 0; i < m.F; i++ )
    {
        // llegir totes les columnes de la fila i
        for( int j = 0; j < m.C; j++ )
        {
            c = getchar();
            ModificarMapa( m, i, j, c );
        }
        //saltar '\n' final de fila i
        c = getchar();
    }
    // Buscar sortida
    m.coords_sortida = BuscarMapa( m, '<' );
    m.coords_joia = BuscarMapa( m, '*' );
}

/* Imprimeix mapa per la sortirda estandar */
void PrintMapa( const Mapa& m )
{
    printf("Mapa: %d x %d\n", m.F, m.C );
    for( int i = 0; i < m.F; i++ )
    {
        for( int j = 0; j < m.C; j++ )
        {
            printf("%c", ConsultarMapa(m,i,j) );
        }
        printf("\n");
    }
    /* NO CAL
    printf( "Sortida = (%d,%d)\n", m.coords_sortida.i, m.coords_sortida.j );
    printf( "Joia = (%d,%d)\n", m.coords_sortida.i, m.coords_sortida.j );
    */
}

// Llegeix 1 enter de l'entrada estandar
int ReadInteger()
{
    int longitud = 0;
    char paraula[128];

    char c = getchar();

    // Saltar espais i returns previs a l'enter
    while( c == ' ' || c == '\n' )
        c = getchar();

    // Llegir caràcters de la paraula que representa l'enter
    while( c != ' ' && c != '\n' && c != EOF )
    {
        paraula[longitud] = c;
        longitud++;
        c = getchar();
    }

    // Posar marca final de paraula
    paraula[longitud] = '\0';

    // Convertir a enter
    return atoi(paraula);
}

#endif //MAPA_H
